@@ -1,4 +1,3 @@
-from __future__ import unicode_literals
 import sys
 import json
 import os
@@ -200,7 +199,6 @@ class Exporter:
         self.export["dumpversion"] = self.DUMPVERSION
         self.export["pythonversion"] = list(sys.version_info)
         self.export["devpi_server"] = server_version
-        self.export["uuid"] = self.xom.config.get_master_uuid()
         for user in self.xom.model.get_userlist():
             userdir = path.join(user.name)
             data = user.get(credentials=True)
@@ -277,7 +275,8 @@ class IndexDump:
             entry = self.exporter.filestore.get_file_entry(link.entrypath)
             if not entry.last_modified:
                 continue
-            assert entry.file_exists(), entry.relpath
+            if not entry.file_exists():
+                fatal(f"The file for {entry.relpath} is missing.")
             relpath = self.exporter.copy_file(
                 entry,
                 self.basedir.join(linkstore.project, link.version, entry.basename))
@@ -302,7 +301,8 @@ class IndexDump:
                               log=tox_link.get_logs())
 
     def add_filedesc(self, type, project, relpath, **kw):
-        assert self.exporter.basepath.join(relpath).check()
+        if not self.exporter.basepath.join(relpath).check():
+            fatal(f"The file for {relpath} is missing.")
         d = kw.copy()
         d["type"] = type
         d["projectname"] = project
@@ -402,9 +402,6 @@ class Importer:
         self.dumpversion = self.import_data["dumpversion"]
         if self.dumpversion not in ("1", "2"):
             fatal(f"incompatible dumpversion: {self.dumpversion!r}")
-        uuid = self.import_data.get("uuid")
-        if uuid is not None:
-            self.xom.config.set_uuid(uuid)
         self.import_users = self.import_data["users"]
         self.import_indexes = self.import_data["indexes"]
         self.display_import_header(path)
@@ -548,7 +545,8 @@ class Importer:
         rel = filedesc["relpath"]
         project = filedesc["projectname"]
         p = self.import_rootdir.join(rel)
-        assert p.check(), p
+        if not p.check():
+            fatal(f"The file at {p} is missing.")
         f = p.open("rb")
         if self.xom.config.hard_links:
             # additional attribute for hard links
@@ -605,7 +603,8 @@ class Importer:
             # it might use a different checksum
             basename = posixpath.basename(filedesc["for_entrypath"])
             link, = linkstore.get_links(basename=basename)
-            link = stage.store_toxresult(link, f)
+            link = stage.store_toxresult(
+                link, f, filename=posixpath.basename(filedesc["relpath"]))
         else:
             fatal("unknown file type: %s" % (type,))
         if link is not None:
