@@ -6,13 +6,12 @@ write Transaction is ongoing.  Each Transaction will see a consistent
 view of key/values referring to the point in time it was started,
 independent from any future changes.
 """
-from __future__ import unicode_literals
-import re
 import contextlib
 import py
 from . import mythread
 from .fileutil import loads
 from .interfaces import IStorageConnection3
+from .keyfs_types import PTypedKey, RelpathInfo, TypedKey
 from .log import threadlog, thread_push_log, thread_pop_log
 from .log import thread_change_log_prefix
 from .readonly import ensure_deeply_readonly
@@ -20,7 +19,6 @@ from .readonly import get_mutable_deepcopy
 from .readonly import is_deeply_readonly
 from .filestore import FileEntry
 from .fileutil import read_int_from_file, write_int_to_file
-import attr
 import time
 
 from devpi_common.types import cached_property
@@ -119,8 +117,8 @@ class TxNotificationThread:
                 self.thread.sleep(1.0)
 
     def get_ixconfig(self, entry, event_serial):
-        user = entry.key.params['user']
-        index = entry.key.params['index']
+        user = entry.user
+        index = entry.index
         if getattr(self, '_get_ixconfig_cache_serial', None) != event_serial:
             self._get_ixconfig_cache = {}
             self._get_ixconfig_cache_serial = event_serial
@@ -395,42 +393,6 @@ class KeyFS(object):
         self.commit_transaction_in_thread()
 
 
-class PTypedKey:
-    __slots__ = ('keyfs', 'name', 'pattern', 'rex_reverse', 'type')
-    rex_braces = re.compile(r'\{(.+?)\}')
-
-    def __init__(self, keyfs, key, type, name):
-        self.keyfs = keyfs
-        self.pattern = py.builtin._totext(key)
-        self.type = type
-        self.name = name
-
-        def repl(match):
-            name = match.group(1)
-            return r'(?P<%s>[^\/]+)' % name
-        rex_pattern = self.pattern.replace("+", r"\+")
-        rex_pattern = self.rex_braces.sub(repl, rex_pattern)
-        self.rex_reverse = re.compile("^" + rex_pattern + "$")
-
-    def __call__(self, **kw):
-        for val in kw.values():
-            if "/" in val:
-                raise ValueError(val)
-        relpath = self.pattern.format(**kw)
-        return TypedKey(self.keyfs, relpath, self.type, self.name,
-                        params=kw)
-
-    def extract_params(self, relpath):
-        m = self.rex_reverse.match(relpath)
-        return m.groupdict() if m is not None else {}
-
-    def on_key_change(self, callback):
-        self.keyfs.notifier.on_key_change(self.name, callback)
-
-    def __repr__(self):
-        return f"<PTypedKey {self.pattern!r} type {self.type.__name__!r}>"
-
-
 class KeyChangeEvent:
     def __init__(self, typedkey, value, at_serial, back_serial):
         self.typedkey = typedkey
@@ -439,68 +401,6 @@ class KeyChangeEvent:
         self.back_serial = back_serial
 
 
-class TypedKey:
-    __slots__ = ('keyfs', 'name', 'params', 'relpath', 'type')
-
-    def __init__(self, keyfs, relpath, type, name, params=None):
-        self.keyfs = keyfs
-        self.relpath = relpath
-        self.type = type
-        self.name = name
-        self.params = params or {}
-
-    def __hash__(self):
-        return hash(self.relpath)
-
-    def __eq__(self, other):
-        return self.relpath == other.relpath
-
-    def __repr__(self):
-        return f"<TypedKey {self.name} {self.type.__name__} {self.relpath}>"
-
-    def get(self, readonly=True):
-        return self.keyfs.tx.get(self, readonly=readonly)
-
-    @property
-    def last_serial(self):
-        try:
-            return self.keyfs.tx.last_serial(self)
-        except KeyError:
-            return None
-
-    def is_dirty(self):
-        return self.keyfs.tx.is_dirty(self)
-
-    @contextlib.contextmanager
-    def update(self):
-        val = self.keyfs.tx.get(self, readonly=False)
-        yield val
-        # no exception, so we can set and thus mark dirty the object
-        self.set(val)
-
-    def set(self, val):
-        if not isinstance(val, self.type):
-            raise TypeError(
-                "%r requires value of type %r, got %r" % (
-                    self.relpath, self.type.__name__, type(val).__name__))
-        self.keyfs.tx.set(self, val)
-
-    def exists(self):
-        return self.keyfs.tx.exists(self)
-
-    def delete(self):
-        return self.keyfs.tx.delete(self)
-
-
-@attr.s(slots=True)
-class RelpathInfo(object):
-    relpath = attr.ib(type=str)
-    keyname = attr.ib(type=str)
-    serial = attr.ib(type=int)
-    back_serial = attr.ib(type=int)
-    value = attr.ib()
-
-
 def get_relpath_at(self, relpath, serial):
     """ Fallback method for legacy storage connections. """
     (keyname, last_serial) = self.db_read_typedkey(relpath)
