@@ -76,11 +76,11 @@ class TagLogger:
 
 class ThreadLog:
     def __getattr__(self, name):
-        return getattr(thread_current_log(), name)
+        return getattr(_thread_current_log(), name)
 
     @contextlib.contextmanager
     def around(self, level, msg, *args):
-        tlog = thread_current_log()
+        tlog = _thread_current_log()
         log = getattr(tlog, level)
         log(msg, *args)
         try:
@@ -102,13 +102,20 @@ def thread_push_log(prefix):
     return tlog
 
 
-def thread_pop_log(prefix=None):
-    if prefix and not threadlocal.taglogger._prefix.rstrip().endswith(prefix):
+def thread_pop_log(prefix):
+    if not threadlocal.taglogger._prefix.rstrip().endswith(prefix):
         raise ValueError("Wrong thread log order, expected %r, saw %r" %
                          (prefix, threadlocal.taglogger._prefix))
     threadlocal.taglogger = threadlocal.taglogger.last
 
 
+def thread_change_log_prefix(prefix, old_prefix):
+    if old_prefix and not threadlocal.taglogger._prefix.rstrip().endswith(old_prefix):
+        raise ValueError("Wrong thread log order, expected %r, saw %r" %
+                         (old_prefix, threadlocal.taglogger._prefix))
+    threadlocal.taglogger._prefix = prefix.rstrip() + " "
+
+
 def thread_clear_log():
     try:
         del threadlocal.taglogger
@@ -116,7 +123,7 @@ def thread_clear_log():
         pass
 
 
-def thread_current_log():
+def _thread_current_log():
     taglogger = getattr(threadlocal, "taglogger", None)
     if taglogger is None:
         taglogger = TagLogger(prefix="NOCTX")
