@@ -3,7 +3,6 @@
 a WSGI server to serve PyPI compatible indexes and a full
 recursive cache of pypi.org packages.
 """
-from __future__ import unicode_literals
 import aiohttp
 import inspect
 import os
@@ -435,7 +434,7 @@ class XOM:
                         # replication protocol
                         return proxy_view_to_master
         return view
-    view_deriver.options = ('is_mutating',)
+    view_deriver.options = ('is_mutating',)  # type: ignore
 
     def create_app(self):
         from devpi_server.middleware import OutsideURLMiddleware
