@@ -5,8 +5,6 @@ toxresult storage.
 
 """
 
-from __future__ import unicode_literals
-
 import asyncio
 import time
 
@@ -23,7 +21,7 @@ from .config import hookimpl
 from .exceptions import lazy_format_exception
 from .filestore import key_from_link
 from .model import BaseStageCustomizer
-from .model import BaseStage, SimplelinkMeta
+from .model import BaseStage
 from .model import ensure_boolean
 from .model import join_links_data
 from .readonly import ensure_deeply_readonly
@@ -568,7 +566,7 @@ class MirrorStage(BaseStage):
             devpi_serial=response.headers.get("X-DEVPI-SERIAL"),
             etag=response.headers.get("ETag")))
 
-    def _update_simplelinks(self, project, info, newlinks):
+    def _update_simplelinks(self, project, info, links, newlinks):
         if self.xom.is_replica():
             # on the replica we wait for the changes to arrive (changes were
             # triggered by our http request above) because we have no direct
@@ -587,7 +585,7 @@ class MirrorStage(BaseStage):
             if links is not None:
                 self.keyfs.tx.on_commit_success(partial(
                     self.cache_retrieve_times.refresh, project, info["etag"]))
-                return links
+                return self.SimpleLinks(links)
             raise self.UpstreamError("no cache links from master for %s" %
                                      project)
         else:
@@ -600,16 +598,19 @@ class MirrorStage(BaseStage):
             maplink = partial(
                 self.filestore.maplink,
                 user=self.user.name, index=self.index, project=project)
-            # calling maplink on the links creates the entries in the database
-            for x in info["releaselinks"]:
-                maplink(x)
+            existing_info = set(links or ())
+            # calling mapkey on the links creates the entries in the database
+            for newinfo, link in zip(newlinks, info["releaselinks"]):
+                if newinfo in existing_info:
+                    continue
+                maplink(link)
             # this stores the simple links info
             self._save_cache_links(
                 project,
                 info["key_hrefs"], info["requires_python"], info["yanked"],
                 info["serial"],
                 info["etag"])
-            return newlinks
+            return self.SimpleLinks(newlinks)
 
     async def _update_simplelinks_in_future(self, newlinks_future, project, lock):
         threadlog.debug("Awaiting simple links for %r", project)
@@ -624,7 +625,7 @@ class MirrorStage(BaseStage):
             (is_expired, links, cache_serial) = self._load_cache_links(project)
             if links is None or set(links) != set(newlinks):
                 # we got changes, so store them
-                self._update_simplelinks(project, info, newlinks)
+                self._update_simplelinks(project, info, links, newlinks)
                 threadlog.debug(
                     "Updated simplelinks for %r in background", project)
             else:
@@ -639,7 +640,7 @@ class MirrorStage(BaseStage):
         exist.
         """
         project = normalize_name(project)
-        lock = self.cache_retrieve_times.lock(project, self.timeout)
+        lock = self.cache_retrieve_times.acquire(project, self.timeout)
         if lock is not None:
             self.keyfs.tx.on_finished(lock.release)
         is_expired, links, cache_serial = self._load_cache_links(project)
@@ -651,7 +652,7 @@ class MirrorStage(BaseStage):
                 threadlog.warn(
                     "serving stale links for %r, waiting for existing request timed out after %s seconds",
                     project, self.timeout)
-                return links
+                return self.SimpleLinks(links, stale=True)
             raise self.UpstreamError(
                 f"timeout after {self.timeout} seconds while getting data for {project!r}")
 
@@ -662,7 +663,7 @@ class MirrorStage(BaseStage):
                 threadlog.debug(
                     "using stale links for %r due to offline mode", project)
                 self._offline_logging.add(project)
-            return links
+            return self.SimpleLinks(links, stale=True)
 
         if links is None:
             is_retrieval_expired = self.cache_retrieve_times.is_expired(
@@ -697,14 +698,14 @@ class MirrorStage(BaseStage):
                 threadlog.warn(
                     "serving stale links for %r, getting data timed out after %s seconds",
                     project, self.timeout)
-                return links
+                return self.SimpleLinks(links, stale=True)
             raise self.UpstreamError(
                 f"timeout after {self.timeout} seconds while getting data for {project!r}")
         except self.UpstreamNotModified as e:
             if links is not None:
                 self.keyfs.tx.on_commit_success(partial(
                     self.cache_retrieve_times.refresh, project, e.etag))
-                return links
+                return self.SimpleLinks(links)
             if e.etag is None:
                 threadlog.error(
                     "server returned 304 Not Modified, but we have no links")
@@ -720,7 +721,7 @@ class MirrorStage(BaseStage):
                 threadlog.warn(
                     "serving stale links, because of exception %s",
                     lazy_format_exception(e))
-                return links
+                return self.SimpleLinks(links, stale=True)
             raise
 
         info = newlinks_future.result()
@@ -730,9 +731,9 @@ class MirrorStage(BaseStage):
         if links is not None and set(links) == set(newlinks):
             # no changes
             self.cache_retrieve_times.refresh(project, info["etag"])
-            return links
+            return self.SimpleLinks(links)
 
-        return self._update_simplelinks(project, info, newlinks)
+        return self._update_simplelinks(project, info, links, newlinks)
 
     def has_project_perstage(self, project):
         project = normalize_name(project)
@@ -742,10 +743,9 @@ class MirrorStage(BaseStage):
 
     def list_versions_perstage(self, project):
         try:
-            return set(x.version
-                       for x in map(SimplelinkMeta, self.get_simplelinks_perstage(project)))
+            return set(x.version for x in self.get_simplelinks_perstage(project))
         except self.UpstreamNotFoundError:
-            return []
+            return set()
 
     def get_last_project_change_serial_perstage(self, project, at_serial=None):
         project = normalize_name(project)
@@ -766,7 +766,7 @@ class MirrorStage(BaseStage):
         # contains whatever this method was called with, which is hopefully
         # the title from the project list
         verdata = {}
-        for sm in map(SimplelinkMeta, self.get_simplelinks_perstage(project)):
+        for sm in self.get_simplelinks_perstage(project):
             link_version = sm.version
             if version == link_version:
                 if not verdata:
@@ -777,7 +777,7 @@ class MirrorStage(BaseStage):
                 if sm.yanked is not None and sm.yanked is not False:
                     verdata['yanked'] = sm.yanked
                 elinks = verdata.setdefault("+elinks", [])
-                entrypath = sm._url.path
+                entrypath = sm.path
                 elinks.append({"rel": "releasefile", "entrypath": entrypath})
         if readonly:
             return ensure_deeply_readonly(verdata)
@@ -837,12 +837,33 @@ class ProjectNamesCache:
         self._etag = etag
 
 
+class ProjectUpdateInnerLock:
+    # this is the object which is stored in the cache
+    # it is needed to add the is_from_current_thread method
+    # and to allow the WeakValueDictionary work correctly
+
+    __slots__ = (
+        '__weakref__', 'acquire', 'locked', 'release', 'thread_ident')
+
+    def __init__(self):
+        self.thread_ident = threading.get_ident()
+        lock = threading.Lock()
+        self.acquire = lock.acquire
+        self.locked = lock.locked
+        self.release = lock.release
+
+    def is_from_current_thread(self):
+        return self.thread_ident == threading.get_ident()
+
+
 class ProjectUpdateLock:
-    def __init__(self, project, project_update_cache):
-        self.lock = project_update_cache._project2lock.setdefault(
-            project, threading.Lock())
+    # this is a wrapper around ProjectUpdateInnerLock to allow
+    # the WeakValueDictionary to work correctly
+
+    def __init__(self, project, lock):
+        self.lock = lock
+        self.locked = lock.locked
         self.project = project
-        self.project_update_cache = project_update_cache
 
     def acquire(self, timeout):
         threadlog.debug("Acquiring lock (%r) for %r", self.lock, self.project)
@@ -854,6 +875,11 @@ class ProjectUpdateLock:
         self.lock = None
         return lock
 
+    def is_from_current_thread(self):
+        if self.lock is not None:
+            return self.lock.is_from_current_thread()
+        return False
+
     def release(self):
         if self.lock is not None:
             self.lock.release()
@@ -892,8 +918,14 @@ class ProjectUpdateCache:
         else:
             self._project2time[project] = (0, etag)
 
-    def lock(self, project, timeout=-1):
-        lock = ProjectUpdateLock(project, self)
+    def acquire(self, project, timeout=-1):
+        lock = ProjectUpdateLock(
+            project,
+            self._project2lock.setdefault(project, ProjectUpdateInnerLock()))
+        if lock.locked() and lock.is_from_current_thread():
+            # remove inner lock, so this is just a dummy
+            lock.lock = None
+            return lock
         if lock.acquire(timeout=timeout):
             return lock
         self._project2lock.pop(project, None)
