@@ -1,4 +1,4 @@
-from __future__ import unicode_literals
+
 import getpass
 import posixpath
 import py
@@ -6,24 +6,18 @@ import re
 import json
 import warnings
 from devpi_common.metadata import get_latest_version
-from devpi_common.metadata import CompareMixin
 from devpi_common.metadata import splitbasename, parse_version
 from devpi_common.url import URL
 from devpi_common.validation import validate_metadata, normalize_name
 from devpi_common.types import ensure_unicode, cached_property, parse_hash_spec
-try:
-    from itertools import zip_longest
-except ImportError:
-    from itertools import izip_longest as zip_longest
-try:
-    from pyramid.authorization import Allow, Authenticated, Everyone
-except ImportError:
-    from pyramid.security import Allow, Authenticated, Everyone
+from functools import total_ordering
+from itertools import zip_longest
+from pyramid.security import Allow, Authenticated, Everyone
 from time import gmtime, strftime
 from .auth import hash_password, verify_and_update_password_hash
 from .config import hookimpl
 from .filestore import FileEntry
-from .filestore import get_file_hash
+from .filestore import get_hash_spec
 from .log import threadlog
 from .readonly import get_mutable_deepcopy
 
@@ -586,6 +580,42 @@ class UnknownCustomizer(BaseStageCustomizer):
     get_principals_for_del_verdata = get_principals_for_index_modify
 
 
+@total_ordering
+class SimpleLinks:
+    __slots__ = ('_links', 'stale')
+
+    def __init__(self, links, stale=False):
+        assert links is not None
+        if isinstance(links, SimpleLinks):
+            self._links = links._links
+            self.stale = links.stale or stale
+        else:
+            self._links = [SimplelinkMeta(x) for x in links]
+            self.stale = stale
+
+    def __iter__(self):
+        return self._links.__iter__()
+
+    def __len__(self):
+        return len(self._links)
+
+    def __eq__(self, other):
+        if isinstance(other, type(self)):
+            other = other._links
+        return self._links == other
+
+    def __lt__(self, other):
+        if isinstance(other, type(self)):
+            other = other._links
+        return self._links < other
+
+    def append(self, item):
+        self._links.append(item)
+
+    def sort(self, *args, **kw):
+        self._links.sort(*args, **kw)
+
+
 class BaseStage(object):
     InvalidIndex = InvalidIndex
     InvalidIndexconfig = InvalidIndexconfig
@@ -597,6 +627,7 @@ class BaseStage(object):
     MissesRegistration = MissesRegistration
     MissesVersion = MissesVersion
     NonVolatile = NonVolatile
+    SimpleLinks = SimpleLinks
 
     def __init__(self, xom, username, index, ixconfig, customizer_cls):
         self.xom = xom
@@ -727,11 +758,14 @@ class BaseStage(object):
         return [self._make_elink(project, link_info)
                 for link_info in self.get_simplelinks_perstage(project)]
 
-    def _make_elink(self, project, link_info):
-        rp = SimplelinkMeta(link_info)
-        linkdict = {"entrypath": rp._url.path, "hash_spec": rp._url.hash_spec,
-                    "require_python": rp.require_python, "yanked": rp.yanked}
-        return ELink(self.filestore, linkdict, project, rp.version)
+    def _make_elink(self, project, link_meta):
+        return ELink(
+            self.filestore, dict(
+                entrypath=link_meta.path,
+                hash_spec=link_meta.hash_spec,
+                require_python=link_meta.require_python,
+                yanked=link_meta.yanked),
+            project, link_meta.version)
 
     def get_linkstore_perstage(self, name, version, readonly=True):
         if self.customizer.readonly and not readonly:
@@ -748,7 +782,7 @@ class BaseStage(object):
         assert len(links) < 2
         return links[0] if links else None
 
-    def store_toxresult(self, link, toxresultdata):
+    def store_toxresult(self, link, toxresultdata, filename=None):
         if self.customizer.readonly:
             raise ReadonlyIndex("index is marked read only")
         linkstore = self.get_linkstore_perstage(link.project, link.version, readonly=False)
@@ -757,7 +791,8 @@ class BaseStage(object):
         return linkstore.new_reflink(
             rel="toxresult",
             content_or_file=toxresultdata,
-            for_entrypath=link)
+            for_entrypath=link,
+            filename=filename)
 
     def get_toxresults(self, link):
         l = []
@@ -853,26 +888,28 @@ class BaseStage(object):
         and "key" is usually the basename of the link or else
         the egg-ID if the link points to an egg.
         """
-        all_links = []
+        all_links = self.SimpleLinks([])
         seen = set()
 
         try:
             for stage, res in self.op_sro_check_mirror_whitelist(
                     "get_simplelinks_perstage", project=project):
+                if res is not None:
+                    res = self.SimpleLinks(res)
+                    all_links.stale = all_links.stale or res.stale
                 iterator = self.customizer.get_simple_links_filter_iter(project, res)
                 if iterator is not None:
                     res = apply_filter_iter(res, iterator)
                 for link_info in res:
-                    key = link_info[0]
+                    key = link_info.key
                     if key not in seen:
                         seen.add(key)
                         all_links.append(link_info)
         except self.UpstreamNotFoundError:
-            return []
+            return self.SimpleLinks([])
 
         if sorted_links:
-            all_links = [(v.key, v.href, v.require_python, v.yanked)
-                        for v in sorted(map(SimplelinkMeta, all_links), reverse=True)]
+            all_links.sort(reverse=True)
         return all_links
 
     def get_whitelist_inheritance(self):
@@ -1272,7 +1309,8 @@ class PrivateStage(BaseStage):
         links = data.get("links", [])
         requires_python = data.get("requires_python", [])
         yanked = []  # PEP 592 isn't supported for private stages yet
-        return join_links_data(links, requires_python, yanked)
+        return self.SimpleLinks(
+            join_links_data(links, requires_python, yanked))
 
     def _regen_simplelinks(self, project_input):
         project = normalize_name(project_input)
@@ -1427,10 +1465,31 @@ def devpiserver_get_stage_customizer_classes():
         ("stage", StageCustomizer)]
 
 
+def linkdictprop(name, default=notset):
+    def fget(self):
+        try:
+            return self.linkdict[name]
+        except KeyError:
+            if default is notset:
+                raise AttributeError(name)
+            return default
+
+    return property(fget)
+
+
 class ELink(object):
     """ model Link using entrypathes for referencing. """
     __slots__ = ('_entry', 'basename', 'filestore', 'linkdict', 'project', 'version')
 
+    _log = linkdictprop("_log")
+    entrypath = linkdictprop("entrypath")
+    for_entrypath = linkdictprop("for_entrypath", default=None)
+    hash_spec = linkdictprop("hash_spec", default="")
+    rel = linkdictprop("rel", default=None)
+    relpath = linkdictprop("entrypath")
+    require_python = linkdictprop("require_python")
+    yanked = linkdictprop("yanked")
+
     def __init__(self, filestore, linkdict, project, version):
         self._entry = notset
         self.filestore = filestore
@@ -1440,12 +1499,12 @@ class ELink(object):
         self.version = version
 
     @property
-    def relpath(self):
-        return self.linkdict["entrypath"]
+    def index(self):
+        return self.entry.index
 
     @property
-    def hash_spec(self):
-        return self.linkdict.get("hash_spec", "")
+    def user(self):
+        return self.entry.user
 
     @property
     def hash_value(self):
@@ -1459,15 +1518,8 @@ class ELink(object):
         hash_algo, hash_value = parse_hash_spec(self.hash_spec)
         if not hash_algo:
             return True
-        return get_file_hash(content_or_file, hash_algo().name) == hash_value
-
-    def __getattr__(self, name):
-        try:
-            return self.linkdict[name]
-        except KeyError:
-            if name in ("for_entrypath", "rel"):
-                return None
-            raise AttributeError(name)
+        return get_hash_spec(
+            content_or_file, hash_algo().name) == self.hash_spec
 
     def __repr__(self):
         return "<ELink rel=%r entrypath=%r>" % (self.rel, self.entrypath)
@@ -1533,16 +1585,17 @@ class LinkStore:
             link.add_log('overwrite', None, count=overwrite + 1)
         return link
 
-    def new_reflink(self, rel, content_or_file, for_entrypath):
+    def new_reflink(self, rel, content_or_file, for_entrypath, filename=None):
         if isinstance(for_entrypath, ELink):
             for_entrypath = for_entrypath.entrypath
         links = self.get_links(entrypath=for_entrypath)
         assert len(links) == 1, f"need exactly one reference, got {links}"
         base_entry = links[0].entry
-        other_reflinks = self.get_links(rel=rel, for_entrypath=for_entrypath)
-        timestamp = strftime("%Y%m%d%H%M%S", gmtime())
-        filename = "%s.%s-%s-%d" % (
-            base_entry.basename, rel, timestamp, len(other_reflinks))
+        if filename is None:
+            other_reflinks = self.get_links(rel=rel, for_entrypath=for_entrypath)
+            timestamp = strftime("%Y%m%d%H%M%S", gmtime())
+            filename = "%s.%s-%s-%d" % (
+                base_entry.basename, rel, timestamp, len(other_reflinks))
         entry = self._create_file_entry(
             filename, content_or_file, ref_hash_spec=base_entry.hash_spec)
         return self._add_link_to_file_entry(
@@ -1607,16 +1660,105 @@ class LinkStore:
                      self.version)
 
 
-class SimplelinkMeta(CompareMixin):
+@total_ordering
+class SimplelinkMeta:
     """ helper class to provide information for items from get_simplelinks() """
-    def __init__(self, link_info):
-        self.key, self.href, self.require_python, self.yanked = link_info
-        self._url = URL(self.href)
-        self.name, self.version, self.ext = splitbasename(self._url.basename, checkarch=False)
 
-    @cached_property
+    __slots__ = (
+        '__basename', '__cmpval', '__ext', '__hash_spec',
+        '__name', '__path', '__url', '__version',
+        'key', 'href', 'require_python', 'yanked')
+
+    def __init__(self, link_info):
+        self.__basename = notset
+        self.__cmpval = notset
+        self.__ext = notset
+        self.__hash_spec = notset
+        self.__name = notset
+        self.__path = notset
+        self.__url = notset
+        self.__version = notset
+        (self.key, self.href, self.require_python, self.yanked) = link_info
+
+    def __eq__(self, other):
+        if isinstance(other, type(self)):
+            other = other.cmpval
+        return self.cmpval == other
+
+    def __lt__(self, other):
+        if isinstance(other, type(self)):
+            other = other.cmpval
+        return self.cmpval < other
+
+    def __getitem__(self, index):
+        warnings.warn(
+            "Item access for SimplelinkMeta is deprecated, "
+            "use attributes instead.",
+            DeprecationWarning)
+        if index == 0:
+            return self.key
+        elif index == 1:
+            return self.href
+        elif index == 2:
+            return self.require_python
+        elif index == 3:
+            return self.yanked
+        raise IndexError(f"{self.__class__.__name__} index out of range")
+
+    def __splitbasename(self):
+        (self.__name, self.__version, self.__ext) = splitbasename(
+            self.basename, checkarch=False)
+
+    def __parse_url(self):
+        url = URL(self.href)
+        self.__basename = url.basename
+        self.__hash_spec = url.hash_spec
+        self.__path = url.path
+
+    @property
+    def basename(self):
+        if self.__basename is notset:
+            self.__parse_url()
+        return self.__basename
+
+    @property
+    def hash_spec(self):
+        if self.__hash_spec is notset:
+            self.__parse_url()
+        return self.__hash_spec
+
+    @property
+    def path(self):
+        if self.__path is notset:
+            self.__parse_url()
+        return self.__path
+
+    @property
+    def name(self):
+        if self.__name is notset:
+            self.__splitbasename()
+        return self.__name
+
+    @property
+    def version(self):
+        if self.__version is notset:
+            self.__splitbasename()
+        return self.__version
+
+    @property
+    def ext(self):
+        if self.__ext is notset:
+            self.__splitbasename()
+        return self.__ext
+
+    @property
     def cmpval(self):
-        return parse_version(self.version), normalize_name(self.name), self.ext
+        if self.__cmpval is notset:
+            self.__cmpval = (
+                parse_version(self.version),
+                normalize_name(self.name),
+                self.ext)
+        return self.__cmpval
 
 
 def make_key_and_href(entry):
