@@ -1,4 +1,3 @@
-from __future__ import unicode_literals
 import contextlib
 import os
 import py
@@ -571,7 +570,9 @@ class PyPIView:
         stage = self.context.stage
         requested_by_installer = is_simple_json_or_requested_by_installer(request)
         try:
-            result = stage.get_simplelinks(project, sorted_links=not requested_by_installer)
+            result = stage.SimpleLinks(
+                stage.get_simplelinks(
+                    project, sorted_links=not requested_by_installer))
         except stage.UpstreamError as e:
             threadlog.error(e.msg)
             abort(request, 502, e.msg)
@@ -603,6 +604,8 @@ class PyPIView:
             serial = stage.key_projsimplelinks(project).get().get("serial")
             if serial is not None and serial > 0:
                 response.headers[str("X-PYPI-LAST-SERIAL")] = str(serial)
+        if result.stale:
+            response.cache_expires()
         return response
 
     def _makeurl_factory(self):
@@ -640,15 +643,15 @@ class PyPIView:
 
         make_url = self._makeurl_factory()
 
-        for key, href, require_python, yanked in result:
-            stage = "/".join(href.split("/", 2)[:2])
-            attribs = 'href="%s"' % make_url(href).url
-            if require_python is not None:
-                attribs += ' data-requires-python="%s"' % escape(require_python)
-            if yanked is not None and yanked is not False:
-                yanked = "" if yanked is True else yanked
+        for link in result:
+            stage = "/".join(link.href.split("/", 2)[:2])
+            attribs = 'href="%s"' % make_url(link.href).url
+            if link.require_python is not None:
+                attribs += ' data-requires-python="%s"' % escape(link.require_python)
+            if link.yanked is not None and link.yanked is not False:
+                yanked = "" if link.yanked is True else link.yanked
                 attribs += ' data-yanked="%s"' % escape(yanked)
-            data = dict(stage=stage, attribs=attribs, key=key)
+            data = dict(stage=stage, attribs=attribs, key=link.key)
             yield '{stage} <a {attribs}>{key}</a><br/>\n'.format(
                 **data).encode('utf-8')
 
@@ -660,16 +663,16 @@ class PyPIView:
         make_url = self._makeurl_factory()
 
         first = True
-        for key, href, require_python, yanked in result:
-            url = make_url(href)
+        for link in result:
+            url = make_url(link.href)
             data = {
-                "filename": key,
+                "filename": link.key,
                 "url": url.url_nofrag,
                 "hashes": {url.hash_type: url.hash_value} if url.hash_type else {}}
-            if require_python is not None:
-                data["requires-python"] = require_python
-            if yanked is not None and yanked is not False:
-                data["yanked"] = yanked
+            if link.require_python is not None:
+                data["requires-python"] = link.require_python
+            if link.yanked is not None and link.yanked is not False:
+                data["yanked"] = link.yanked
             info = json.dumps(data, indent=None, sort_keys=False)
             if first:
                 yield f'{info}'.encode("utf-8")
@@ -1628,9 +1631,7 @@ def iter_cache_remote_file(xom, entry, url):
                 f, r.headers.get("last-modified", None),
                 hash_spec=hash_spec)
             if entry.project:
-                stage = xom.model.getstage(
-                    entry.key.params['user'],
-                    entry.key.params['index'])
+                stage = xom.model.getstage(entry.user, entry.index)
                 # for mirror indexes this makes sure the project is in the database
                 # as soon as a file was fetched
                 stage.add_project_name(entry.project)
