@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 import os
 import sys
 import py
@@ -10,7 +8,6 @@ from devpi_server.importexport import Exporter
 from devpi_server.importexport import IndexTree
 from devpi_server.importexport import do_export, do_import
 from devpi_server.main import Fatal
-from devpi_server.readonly import get_mutable_deepcopy
 from devpi_common.archive import Archive, zip_dict
 from devpi_common.metadata import Version
 from devpi_common.url import URL
@@ -203,7 +200,7 @@ class TestImportExport:
                 set_state_version(self.mapp1.xom.config)
                 self.options = options
 
-            def export(self, initnodeinfo=True):
+            def export(self):
                 from devpi_server.importexport import export
                 argv = [
                     "devpi-export",
@@ -317,10 +314,10 @@ class TestImportExport:
             '/user1/dev6/+f/dab/741b6289e7dcc/pkg1-1.0.tar.gz']
 
     def test_uuid(self, impexp):
+        nodeinfo1 = impexp.mapp1.xom.config.nodeinfo
         impexp.export()
-        mapp2 = impexp.new_import()
-        assert mapp2.xom.config.nodeinfo["uuid"] == \
-              impexp.mapp1.xom.config.nodeinfo["uuid"]
+        nodeinfo2 = impexp.new_import().xom.config.nodeinfo
+        assert nodeinfo1["uuid"] != nodeinfo2["uuid"]
 
     def test_two_indexes_inheriting(self, impexp):
         mapp1 = impexp.mapp1
@@ -474,8 +471,9 @@ class TestImportExport:
             stage.offline = True
             projects = stage.list_projects_perstage()
             assert projects == {'package': 'package'}
-            links = sorted(get_mutable_deepcopy(
-                stage.get_simplelinks_perstage("package")))
+            links = sorted(
+                (x.key, x.href, x.require_python, x.yanked)
+                for x in stage.get_simplelinks_perstage("package"))
             assert links == [
                 ('package-1.1.zip', 'root/pypi/+f/a66/5a45920422f9d/package-1.1.zip', None, None),
                 ('package-1.2.zip', 'root/pypi/+f/b3a/8e0e1f9ab1bfe/package-1.2.zip', None, ""),
@@ -487,7 +485,7 @@ class TestImportExport:
             stage = mapp.xom.model.getstage('root/pypi')
             stage.offline = True
             (link,) = stage.get_simplelinks_perstage("dddttt")
-            link = stage.get_link_from_entrypath(link[1])
+            link = stage.get_link_from_entrypath(link.href)
             assert link.project == "dddttt"
             assert link.version == "0.1.dev1"
             assert link.relpath == 'root/pypi/+f/100/7f0cd10aaa290/dddttt-0.1.dev1.tar.gz'
@@ -582,6 +580,17 @@ class TestImportExport:
             assert history_log[0]['who'] == 'user1'
             assert history_log[0]['dst'] == 'user1/dev'
 
+    def test_import_with_old_toxresult_naming_scheme(self, impexp):
+        mapp = impexp.import_testdata('toxresult_naming_scheme')
+        with mapp.xom.keyfs.transaction(write=False):
+            stage = mapp.xom.model.getstage("root/dev")
+            ls = stage.get_linkstore_perstage("hello", "0.9")
+            (release,) = ls.get_links(rel="releasefile")
+            (toxresult,) = ls.get_links(rel="toxresult")
+            assert toxresult.for_entrypath == release.relpath
+            # we expect the name from the imported data
+            assert toxresult.basename == "hello-0.9.tar.gz.toxresult0"
+
     def test_import_without_history_log(self, impexp, tox_result_data):
         DUMP_FILE = {
           "users": {
@@ -844,23 +853,6 @@ class TestImportExport:
         mapp2 = impexp.new_import()
         mapp2.login("exp", "pass")
 
-    def test_import_replica_preserves_master_uuid(self, impexp):
-        mapp1 = impexp.mapp1
-        mapp1.create_and_login_user("exp", "pass")
-        # fake it's a replica
-        # delete cached value
-        del mapp1.xom.config._master_url
-        mapp1.xom.config.nodeinfo["role"] = "replica"
-        mapp1.xom.config.nodeinfo["masterurl"] = "http://xyz"
-        mapp1.xom.config.init_nodeinfo()
-        mapp1.xom.config.set_master_uuid("mm")
-        mapp1.xom.config.set_uuid("1111")
-        impexp.export(initnodeinfo=False)
-        mapp2 = impexp.new_import()
-        assert mapp2.xom.config.nodeinfo["role"] == "standalone"
-        assert mapp2.xom.config.get_master_uuid() == "mm"
-        assert mapp2.xom.config.nodeinfo["uuid"] == "mm"
-
     def test_docs_are_preserved(self, impexp):
         mapp1 = impexp.mapp1
         api = mapp1.create_and_use()
