@@ -65,7 +65,7 @@ class TestRenameFileLogic:
         with pytest.raises(RuntimeError):
             with keyfs.transaction() as tx:
                 tx.conn.io_file_set('foo', b'foo')
-                tmppath = tx.conn.dirty_files[keyfs.basedir.join('foo')].tmppath
+                tmppath = tx.conn.dirty_files[keyfs.basedir.join('foo').strpath].tmppath
                 assert os.path.exists(tmppath)
                 # abort transaction
                 raise RuntimeError
