@@ -29,9 +29,12 @@ def config(gentmp):
 
 
 def test_pkgresources_version_matches_init():
-    import pkg_resources
+    try:
+        import importlib.metadata as importlib_metadata
+    except ImportError:
+        import importlib_metadata as importlib_metadata
     ver = devpi_server.__version__
-    assert pkg_resources.get_distribution("devpi_server").version == ver
+    assert importlib_metadata.version("devpi-server") == ver
 
 
 def test_version(capfd):
@@ -386,7 +389,7 @@ def test_root_passwd_hash_option(gentmp, makexom, storage_info):
     init(argv=argv)
     xom = makexom(opts=("--serverdir", serverdir))
     # we allow writes, because the hash might be upgraded
-    # this happend after passlib 1.7.2 made argon2id the default
+    # this happened after passlib 1.7.2 made argon2id the default
     with xom.keyfs.transaction(write=True):
         user = xom.model.get_user('root')
         assert not user.validate("")
