@@ -1,4 +1,3 @@
-from __future__ import unicode_literals
 import requests.exceptions
 import time
 import hashlib
@@ -563,6 +562,33 @@ class TestExtPYPIDB:
         assert call['extra_headers']['If-None-Match'] == '"foo"'
         assert pypistage.cache_retrieve_times.get_etag("foo") == '"bar"'
 
+    @pytest.mark.notransaction
+    def test_stale_nocache(self, pypistage, testapp):
+        pypistage.mock_simple("foo", text='<a href="foo-1.0.tar.gz"</a>')
+        r = testapp.xget(200, '/root/pypi/+simple/foo/')
+        assert 'Cache-Control' not in r.headers
+        assert 'Expires' not in r.headers
+        assert 'Pragma' not in r.headers
+        pypistage.mock_simple("foo", status_code=502)
+        r = testapp.xget(200, '/root/pypi/+simple/foo/')
+        assert 'max-age=0' in r.headers['Cache-Control']
+        assert 'Expires' in r.headers
+        assert r.headers['Pragma'] == 'no-cache'
+
+    @pytest.mark.notransaction
+    def test_stale_nocache_inherit(self, mapp, pypistage, testapp):
+        api = mapp.create_and_use(indexconfig=dict(bases='root/pypi'))
+        pypistage.mock_simple("foo", text='<a href="foo-1.0.tar.gz"</a>')
+        r = testapp.xget(200, f'{api.index}/+simple/foo/')
+        assert 'Cache-Control' not in r.headers
+        assert 'Expires' not in r.headers
+        assert 'Pragma' not in r.headers
+        pypistage.mock_simple("foo", status_code=502)
+        r = testapp.xget(200, f'{api.index}/+simple/foo/')
+        assert 'max-age=0' in r.headers['Cache-Control']
+        assert 'Expires' in r.headers
+        assert r.headers['Pragma'] == 'no-cache'
+
 
 class TestMirrorStageprojects:
     def test_get_remote_projects(self, pypistage):
@@ -781,16 +807,16 @@ class TestMirrorStageprojects:
             pypistage = mapp.xom.model.getstage("root/pypi")
             assert pypistage.mirror_url == url
             (link,) = pypistage.get_simplelinks("pkg")
-        assert "foo" not in link[1]
-        assert "bar" not in link[1]
-        assert "/+e/" in link[1]
+        assert "foo" not in link.href
+        assert "bar" not in link.href
+        assert "/+e/" in link.href
         ((path, headers), *_) = simpypi.requests
         (kind, data) = headers["Authorization"].split(None, 1)
         assert kind.lower() == "basic"
         assert base64.b64decode(data) == b"foo:bar"
         simpypi.clear_requests()
         assert not simpypi.requests
-        r = testapp.xget(200, f"/{link[1]}")
+        r = testapp.xget(200, f"/{link.href}")
         assert r.body == content
         ((path, headers),) = simpypi.requests
         (kind, data) = headers["Authorization"].split(None, 1)
@@ -821,16 +847,16 @@ class TestMirrorStageprojects:
             pypistage = mapp.xom.model.getstage("root/pypi")
             assert pypistage.mirror_url == url
             (link,) = pypistage.get_simplelinks("pkg")
-        assert "foo" not in link[1]
-        assert "bar" not in link[1]
-        assert "/+f/" in link[1]
+        assert "foo" not in link.href
+        assert "bar" not in link.href
+        assert "/+f/" in link.href
         ((path, headers), *_) = simpypi.requests
         (kind, data) = headers["Authorization"].split(None, 1)
         assert kind.lower() == "basic"
         assert base64.b64decode(data) == b"foo:bar"
         simpypi.clear_requests()
         assert not simpypi.requests
-        r = testapp.xget(200, f"/{link[1]}")
+        r = testapp.xget(200, f"/{link.href}")
         assert r.body == content
         ((path, headers),) = simpypi.requests
         (kind, data) = headers["Authorization"].split(None, 1)
@@ -1085,7 +1111,7 @@ def test_get_last_project_change_serial_perstage(xom, pypistage):
         assert pypistage.get_last_project_change_serial_perstage('pkg') == -1
         pypistage.mock_simple("pkg", pkgver="pkg-1.0.zip")
         (link,) = pypistage.get_simplelinks_perstage('pkg')
-        assert link[0] == 'pkg-1.0.zip'
+        assert link.key == 'pkg-1.0.zip'
     with xom.keyfs.transaction() as tx:
         # the new project has been updated in the db
         assert tx.at_serial == (first_serial + 2)
@@ -1095,7 +1121,7 @@ def test_get_last_project_change_serial_perstage(xom, pypistage):
         assert tx.at_serial == (first_serial + 2)
         pypistage.mock_simple("other", pkgver="other-1.0.zip")
         (link,) = pypistage.get_simplelinks_perstage('other')
-        assert link[0] == 'other-1.0.zip'
+        assert link.key == 'other-1.0.zip'
     with xom.keyfs.transaction() as tx:
         # the new project has been updated in the db
         assert tx.at_serial == (first_serial + 3)
